/**
 * This file is part of my seminar project.
 * Following code, along with other files in this project,
 * realizes the visualization of A* algorithm.
 * 
 * @author Teon Banek
 */
package hr.fer.seminar;

import java.awt.BorderLayout;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowEvent;
import java.awt.event.WindowListener;

import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JSlider;
import javax.swing.SwingUtilities;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

/**
 * This class is the root frame of the whole application. The application is
 * visual representation of the workings of A* path-finding algorithm. User can
 * set blockable tiles on predefined size map of tiles as well as set start and
 * goal locations for A* algorithm. Certain other algorithm parameters may be
 * changed, and upon starting the algorithm the result can be observed.
 * 
 * @author Teon Banek
 * @version 1.0
 */
public class App extends JFrame {

	private static final long serialVersionUID = 1L;

	private JButton toggleDirectionsButton = new JButton("4-way pathfinding");
	private SliderLabel delayLabel = new SliderLabel();

	/**
	 * Grid based graph where the pathfinding will occur.
	 */
	private TileMap tileMap = null;

	/**
	 * Reference to pathfinder thread.
	 */
	private Pathfinder aStar = null;

	/**
	 * Default constructor that initializes the frame.
	 */
	public App() {

		setResizable(false);
		setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);
		setTitle("A* pathfinding V1.0");

		initGui();
	}

	/**
	 * Sets up the GUI of this application.
	 */
	private void initGui() {
		JButton resetButton = new JButton("Reset");
		JButton startButton = new JButton("Pick start");
		JButton goalButton = new JButton("Pick goal");
		JButton startAlgorithmButton = new JButton("Begin pathfinding");
		JButton nextStepButton = new JButton("Next step");
		JButton toggleSkipButton = new JButton("Skip steps");

		ActionListener bal = new ButtonActionListener();

		resetButton.setActionCommand("reset");
		resetButton.addActionListener(bal);
		startButton.setActionCommand("pick start");
		startButton.addActionListener(bal);
		goalButton.setActionCommand("pick goal");
		goalButton.addActionListener(bal);
		startAlgorithmButton.setActionCommand("begin");
		startAlgorithmButton.addActionListener(bal);
		nextStepButton.setActionCommand("next");
		nextStepButton.addActionListener(bal);
		toggleSkipButton.setActionCommand("skip");
		toggleSkipButton.addActionListener(bal);
		toggleDirectionsButton.setActionCommand("directions");
		toggleDirectionsButton.addActionListener(bal);

		JSlider delaySlider = new JSlider(JSlider.HORIZONTAL, 0, 1000, 0);
		delaySlider.setMajorTickSpacing(250);
		delaySlider.setMinorTickSpacing(100);
		delaySlider.setPaintTicks(true);
		delaySlider.setPaintLabels(true);
		delaySlider.addChangeListener(delayLabel);

		JPanel delayPanel = new JPanel();
		delayPanel.setLayout(new BorderLayout(0, 4));
		delayPanel.add(delayLabel, BorderLayout.PAGE_START);
		delayPanel.add(delaySlider, BorderLayout.CENTER);

		JPanel panel = new JPanel();
		JPanel buttonPanel = new JPanel();
		tileMap = new TileMap(20, 20);
		panel.add(tileMap);

		buttonPanel.add(delayPanel);
		buttonPanel.add(toggleDirectionsButton);
		buttonPanel.add(startAlgorithmButton);
		buttonPanel.add(nextStepButton);
		buttonPanel.add(toggleSkipButton);
		buttonPanel.add(resetButton);
		buttonPanel.add(startButton);
		buttonPanel.add(goalButton);
		buttonPanel.setLayout(new GridLayout(0, 1, 0, 10));
		panel.add(buttonPanel);
		add(panel);

		addWindowListener(new WindowListener() {
			@Override
			public void windowOpened(WindowEvent e) {
			}

			@Override
			public void windowIconified(WindowEvent e) {
			}

			@Override
			public void windowDeiconified(WindowEvent e) {
			}

			@Override
			public void windowDeactivated(WindowEvent e) {
			}

			@Override
			public void windowClosing(WindowEvent e) {
			}

			@Override
			public void windowClosed(WindowEvent e) {
				if (aStar != null) {
					aStar.setRunning(false);
				}
			}

			@Override
			public void windowActivated(WindowEvent e) {
			}
		});
	}

	/**
	 * Begin the application.
	 * 
	 * @param args
	 *            - not used.
	 */
	public static void main(String[] args) {
		SwingUtilities.invokeLater(new Runnable() {

			@Override
			public void run() {
				App app = new App();
				app.pack();
				app.setVisible(true);
			}
		});
	}

	/**
	 * This class listens to actions performed on buttons in {@link App}.
	 * 
	 * @author Teon Banek
	 * 
	 */
	private class ButtonActionListener implements ActionListener {

		private boolean moveDiagonally = false;

		@Override
		public void actionPerformed(ActionEvent e) {
			String cmd = e.getActionCommand();
			if (cmd == "reset") {
				tileMap.resetMap();
				tileMap.updateUI();
				if (aStar != null) {
					aStar.setRunning(false);
				}
				aStar = null;
			}
			if (cmd == "pick start") {
				if (aStar != null) {
					aStar.setRunning(false);
					tileMap.resetPathFlags();
					tileMap.updateUI();
				}
				tileMap.setPickStart(true);
			}
			if (cmd == "pick goal") {
				if (aStar != null) {
					aStar.setRunning(false);
					tileMap.resetPathFlags();
					tileMap.updateUI();
				}
				tileMap.setPickGoal(true);
			}
			if (cmd == "begin") {
				Tile start = tileMap.getStartTile();
				Tile goal = tileMap.getGoalTile();
				if (start != null && goal != null) {
					if (aStar != null) {
						aStar.setRunning(false);
						tileMap.resetPathFlags();
						tileMap.updateUI();
					}
					aStar = new Pathfinder(start, goal, tileMap,
							moveDiagonally, delayLabel.getDelay());
					aStar.start();
				}
			}
			if (cmd == "directions") {
				moveDiagonally = !moveDiagonally;
				if (moveDiagonally) {
					toggleDirectionsButton.setText("8-way pathfinding");
				} else {
					toggleDirectionsButton.setText("4-way pathfinding");
				}
			}
			if (cmd == "next") {
				if (aStar != null) {
					aStar.setNextStep(true);
				}
			}
			if (cmd == "skip") {
				if (aStar != null) {
					aStar.setFastForwarding(true);
					aStar.setDelay(delayLabel.getDelay());
				}
			}
		}
	}

	/**
	 * This class is a label of a slider. It listens to slider's value and
	 * changes text accordingly.
	 * 
	 * @author Teon Banek
	 * 
	 */
	private class SliderLabel extends JLabel implements ChangeListener {

		private static final long serialVersionUID = 1L;

		private int delay = 0;

		/**
		 * Default constructor.
		 */
		public SliderLabel() {
			setText(" Delay for: " + delay + "ms");
		}

		@Override
		public void stateChanged(ChangeEvent e) {
			int newDelay = ((JSlider) e.getSource()).getValue();
			if (newDelay != delay) {
				delay = newDelay;
				setText(" Delay for: " + delay + "ms");
			}
		}

		/**
		 * @return the delay
		 */
		public int getDelay() {
			return delay;
		}
	}
}
