/**
 * This file is part of my seminar project.
 * Following code, along with other files in this project,
 * realizes the visualization of A* algorithm.
 * 
 * @author Teon Banek
 */

package hr.fer.seminar;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.event.MouseAdapter;
import java.awt.image.BufferedImage;
import java.io.IOException;
import java.net.URL;

import javax.imageio.ImageIO;
import javax.swing.BorderFactory;
import javax.swing.JPanel;

/**
 * This class creates a tile map of {@link Tile}. Events on tile map are then
 * performed on tiles.
 * 
 * @author Teon Banek
 * 
 */
public class TileMap extends JPanel {

	private static final long serialVersionUID = 1L;

	private BufferedImage tileset = null;
	private Tile[] tiles = null;
	private int mapWidth;
	private int mapHeight;

	/**
	 * Flag: has the user requested to pick the starting tile.
	 */
	private boolean pickStart = false;

	/**
	 * Flag: has the user requested to pick the goal tile.
	 */
	private boolean pickGoal = false;

	private Tile startTile = null;
	private Tile goalTile = null;

	/**
	 * Initializes with given width and height of a map. This IS NOT pixel size.
	 * 
	 * @param width
	 *            of map.
	 * @param height
	 *            of map.
	 */
	public TileMap(int width, int height) {
		setLayout(null);
		this.mapWidth = width;
		this.mapHeight = height;
		tiles = new Tile[width * height];
		setBorder(BorderFactory.createLineBorder(Color.black));

		MouseAdapter mouseAdapter = new TileMapMouseListener(this);
		addMouseListener(mouseAdapter);
		addMouseMotionListener(mouseAdapter);

		try {
			URL url = this.getClass().getResource("/gfx/tileset.png");
			tileset = ImageIO.read(url);
		} catch (IOException e) {
			System.err
					.println("Unable to read image resource: ./gfx/tileset.png");
			System.exit(-1);
		}
		createMap();
	}

	/**
	 * Gets the tile at position (X,Y) in this map of tiles.
	 * 
	 * @param x
	 *            - x coordinate in tile map.
	 * @param y
	 *            - y coordinate in tile map.
	 * @return {@link Tile} or null if no tile found.
	 */
	public Tile getTile(int x, int y) {
		int location = y * mapWidth + x;
		if (x < 0 || x >= mapWidth || y < 0 || y >= mapHeight) {
			return null;
		}
		return tiles[location];
	}

	/**
	 * Reset the map of tiles to default state.
	 */
	public void resetMap() {
		for (Tile tile : tiles) {
			tile.resetFlags();
			tile.labelF.setText("");
		}
		startTile = null;
		goalTile = null;
	}

	/**
	 * Resets only pathfinder related flags on this map.
	 */
	public void resetPathFlags() {
		for (Tile tile : tiles) {
			tile.resetPathFlags();
		}
	}

	@Override
	protected void paintComponent(Graphics g) {
		super.paintComponent(g);
		for (Tile tile : tiles) {
			tile.paint(g);
		}
	}

	@Override
	public Dimension getPreferredSize() {
		return new Dimension(mapWidth * Tile.WIDTH, mapHeight * Tile.HEIGHT);
	}

	/**
	 * Initializes the array of tiles.
	 */
	private void createMap() {
		for (int y = 0; y < mapHeight; y++) {
			for (int x = 0; x < mapWidth; x++) {
				tiles[y * mapWidth + x] = new Tile(tileset, x, y);
				add(tiles[y * mapWidth + x].labelF);
				tiles[y * mapWidth + x].labelF.setBounds(x * Tile.WIDTH, y
						* Tile.HEIGHT, Tile.WIDTH + 4, Tile.HEIGHT / 2);
			}
		}
	}

	/**
	 * @return the mapWidth
	 */
	public int getMapWidth() {
		return mapWidth;
	}

	/**
	 * @return the mapHeight
	 */
	public int getMapHeight() {
		return mapHeight;
	}

	/**
	 * @return the pickStart
	 */
	public boolean isPickStart() {
		return pickStart;
	}

	/**
	 * @param pickStart
	 *            the pickStart to set
	 */
	public void setPickStart(boolean pickStart) {
		this.pickStart = pickStart;
	}

	/**
	 * @return the pickGoal
	 */
	public boolean isPickGoal() {
		return pickGoal;
	}

	/**
	 * @param pickGoal
	 *            the pickGoal to set
	 */
	public void setPickGoal(boolean pickGoal) {
		this.pickGoal = pickGoal;
	}

	/**
	 * @return the startTile
	 */
	public Tile getStartTile() {
		return startTile;
	}

	/**
	 * Sets the new start tile overriding the old one.
	 * 
	 * @param x
	 *            coordinate on this map.
	 * @param y
	 *            coordinate on this map.
	 */
	public void setStartTile(int x, int y) {
		if (startTile != null) {
			startTile.setStart(false);
		}
		startTile = getTile(x, y);
		if (startTile != null) {
			if (startTile == goalTile) {
				goalTile.setGoal(false);
				goalTile = null;
			}
			startTile.setStart(true);
			pickStart = false;
			updateUI();
		}
	}

	/**
	 * @return the goalTile
	 */
	public Tile getGoalTile() {
		return goalTile;
	}

	/**
	 * Sets the new goal tile overriding the previous one.
	 * 
	 * @param x
	 *            coordinate on this map.
	 * @param y
	 *            coordinate on this map.
	 */
	public void setGoalTile(int x, int y) {
		if (goalTile != null) {
			goalTile.setGoal(false);
		}
		goalTile = getTile(x, y);
		if (goalTile != null) {
			if (goalTile == startTile) {
				startTile.setStart(false);
				startTile = null;
			}
			goalTile.setGoal(true);
			pickGoal = false;
			updateUI();
		}
	}
}
